#####Incorporate Data_Aggregate.RData#####

#####Constants#####
n <- 26	#sample size
u_bound <- 2.8  	#upper bound of domain
l_bound <- 0.2	#lower bound of domain
eta <- u_bound - l_bound	#domain
wid <- .001		#width must be the same.
z <- seq(l_bound, u_bound, wid)
number.z <- length(z)
l_number <- l_bound/wid + 1
u_number <- u_bound/wid + 1
###################

#####The domain is modified#####
y_disc <- as.list(rep(NA, n))
const.y <- rep(NA, n)
for(i in 1:n){
	const.y[i] <- wid*sum((1/2)*sum(temp_y_disc[[i]][l_number], temp_y_disc[[i]][u_number]), sum(temp_y_disc[[i]][(l_number + 1):(u_number -1)]))
	y_disc[[i]] <- temp_y_disc[[i]][l_number:u_number]/const.y[i] 
	}
#################################

#####Explanatory Variables#####
x_1 <- rep(1, n)
x_2 <- read.csv("GDP.csv")
x_3 <- rep(0, 26)
x_3[17:26] <- 1	#dummy variable
temp <- cbind(x_1, x_2, x_3)
X_ex <- as.matrix(temp)
temp_ast <- cbind(x_1, x_3)
X_ex_ast <- as.matrix(temp_ast)
###############################

#####The Data are transformed into matrix form.#####
y_matrix <- matrix(NA, ncol=length(z), nrow=n)
for(i in 1:n){
	y_matrix[i,] <- y_disc[[i]]
	}
####################################################

#####Figure 3 in the main text#####
#number.col <- length(z)
#number.row <- n
#jet.colors <- colorRampPalette( c("blue", "green", "yellow", "red") )	#Create a function interpolating colors in the range of specified colors
#nbcol <- 100
#color <- jet.colors(nbcol)
#zfacet <- y_matrix[-1, -1] + y_matrix[-1, -number.col] + y_matrix[-number.row, -1] + y_matrix[-number.row, -number.col] 	#Compute the z-value at the facet centres
#facetcol <- cut(zfacet, nbcol)	#Recode facet z-values into color indices
#par(mar= c(0.1, 0.9, 0.1, 0.9))
#persp(seq(1989, 2014, 1), z, y_matrix, col = color[facetcol], phi = 30, theta = 50, border=NA, ticktype="detailed", expand=0.8, xlab="Year", ylab="z", zlab="Density", zlim=c(0,1.4))
#par(mar=c(5, 4, 4, 2) + 0.1)
####################################

#####Clr transformation#####
clr <- function(y){
	log_y <- log(y)
	Int_y <- wid*sum((1/2)*sum(log_y[1], log_y[number.z]), sum(log_y[2:(number.z -1)]))
	result <- log_y - (1/eta)*Int_y
	return(result)
}
clr_y_matrix <- matrix(NA, ncol=length(z), nrow=n)
for(i in 1:n){
	clr_y_matrix[i,] <- clr(y_disc[[i]])
		}
############################

#####The least square estimate and the distance#####
norm_L2_disc <- function(x){		#L^2 norm 
	x <- x^2
	integ <- wid*sum((1/2)*sum(x[1], x[number.z]), sum(x[2:(number.z -1)]))
	norm <- integ^(1/2)
	return(norm)
	}
est <- function(Y_mat, X_var, X_var_ast){			#The least square esitmation
	beta_disc <- solve(crossprod(X_var), crossprod(X_var, Y_mat))
	beta_ast_disc <- solve(crossprod(X_var_ast), crossprod(X_var_ast, Y_mat))
	clr_y_hat_disc <- X_var%*%beta_disc
	clr_y_hat_ast_disc <- X_var_ast%*%beta_ast_disc
	error_disc <- matrix(0, ncol=length(z), nrow=n)
	for(i in 1:n){
		error_disc[i, ] <- Y_mat[i, ] - clr_y_hat_disc[i, ]
	}
	error_ast_disc <- matrix(0, ncol=length(z), nrow=n)
	for(i in 1:n){
		error_ast_disc[i, ] <- Y_mat[i, ] - clr_y_hat_ast_disc[i, ]
	}
	norm_error <- numeric(n)
	for(i in 1:n){
		norm_error[i] <- norm_L2_disc(error_disc[i,])^2
	}
	sum_error <- sum(norm_error)
	norm_error_ast <- numeric(n)
	for(i in 1:n){
		norm_error_ast[i] <- norm_L2_disc(error_ast_disc[i, ])^2
	}
	sum_ast_error <- sum(norm_error_ast)
	test_lambda <- sum_ast_error - sum_error
	return(list(beta_disc=beta_disc, clr_y_hat_disc=clr_y_hat_disc, beta_ast_disc=beta_ast_disc, clr_y_hat_ast_disc=clr_y_hat_ast_disc, 
			error_disc=error_disc, error_ast_disc=error_ast_disc, sum_error=sum_error, sum_ast_error=sum_ast_error, test_lambda=test_lambda))	#test_lambda is the test statistics in Proposition 3.
}
original_est_res <- est(clr_y_matrix, X_ex, X_ex_ast)
########################################################

#####Bootstrap methods#####
set.seed(123)
B <- 1000			#Number of bootstrap samples
boot_lambda <- numeric(B)
for(b in 1:B) {
	i <- sample(1:n, size = n, replace = TRUE)	#randomly select the indices
	new_residual <- original_est_res$error_ast_disc[i, ] 
	boot_y_disc <- original_est_res$clr_y_hat_ast_disc + new_residual
	boot_est <- est(boot_y_disc, X_ex, X_ex_ast)
	boot_lambda[b] <- boot_est$test_lambda		#substitute test statistics
}
###Figure 5(a)###
#hist(boot_lambda, prob = TRUE, breaks="Scott", main="", xlab=expression(Lambda))
##############

############################

#####Variance tests, Asymptotic theory#####
error_z <- t(original_est_res$error_disc)		#T by n matrix is required. Transpose. 
library(fda)		#To calculate the eigenvalues, fda method is used. 
bspline_basis <- create.bspline.basis(c(l_bound, u_bound), nbasis=30, norder=4)
errorfd <- smooth.basis(z, error_z, bspline_basis)$fd
eigensystem <- pca.fd(errorfd, nharm=10, centerfns=FALSE)
eig_lambda <- eigensystem$values
eig_prop <- eigensystem$varprop
library(momentchi2)	#Chi-squared distribution
cof_vec <- c(original_est_res$test_lambda)
asy_per <- hbe(eig_lambda[1:20], cof_vec) 
############################################

#####Test results#####
asy_per
hbe(eig_lambda[1:20], 0.0435);hbe(eig_lambda[1:20], 0.0322);hbe(eig_lambda[1:20], 0.0271)
quantile(boot_lambda, 0.99);quantile(boot_lambda, 0.97);quantile(boot_lambda, 0.95)
######################

#####Graph Representation of Results#####
###Figure 4(a)###
#plot(z, original_est_res$beta_disc[2,], col="black", type="l", lwd=2, main="", ylab="", xlab="z")
###Figure 4(b)###
mean <- mean(as.matrix(x_2))
sd <- sd(as.matrix(x_2))
pos <- mean + 2*sd
neg <- mean - 2*sd
X_mean <- matrix(c(1, mean, 0), nrow=1, ncol=3)
X_pos_GR <- matrix(c(1, pos, 0), nrow=1, ncol=3)
X_neg_GR <- matrix(c(1, neg, 0), nrow=1, ncol=3) 
clr_y_mean_disc <- X_mean%*%original_est_res$beta_disc
clr_y_pos_GR_disc <- X_pos_GR%*%original_est_res$beta_disc
clr_y_neg_GR_disc <- X_neg_GR%*%original_est_res$beta_disc
#plot(z, clr_y_mean_disc, xlim=c(l_bound, u_bound), type="l", lwd=2, ylab="", xlab="z")
#lines(z, clr_y_pos_GR_disc, lty=2, lwd=2, col="red")
#lines(z, clr_y_neg_GR_disc, lty=4, lwd=2, col="blue")
###Figure 4(c)###
Density_mean <- exp(clr_y_mean_disc)
nor_mean <- wid*sum((1/2)*sum(Density_mean[1], Density_mean[number.z]), sum(Density_mean[2:(number.z -1)]))
Density_pos_GR <- exp(clr_y_pos_GR_disc)
nor_pos <- wid*sum((1/2)*sum(Density_pos_GR[1], Density_pos_GR[number.z]), sum(Density_pos_GR[2:(number.z -1)]))
Density_neg_GR <- exp(clr_y_neg_GR_disc)
nor_neg <- wid*sum((1/2)*sum(Density_neg_GR[1], Density_neg_GR[number.z]), sum(Density_neg_GR[2:(number.z -1)]))
#plot(z, Density_mean/nor_mean, lty=1, type="l", lwd=2, ylab="Density")
#lines(z, Density_pos_GR/nor_pos, lty=2, type="l", col="red", lwd=2)
#lines(z, Density_neg_GR/nor_neg, lty=4, type="l", col="blue", lwd=2)
#############################

#####Gini coefficient#####
frequency_1 <- Density_mean/nor_mean
Gini.index <- function(frequency){
	Inv <- approxfun(cumsum(frequency*0.001), z)	#note width 0.001. 
	mean_x <- integrate(Inv, lower = 0.001, upper = 1)
	n_points <- 50
	p_width <- 1/n_points 
	Lorent <- rep(1, n_points)
	points <- seq(p_width, 1, p_width)
	for(i in 1:n_points){
		Lorent[i] <- integrate(Inv, lower = 0.001, upper = points[i])$value/mean_x$value
		}
	return(2*sum(points - Lorent)*p_width)
	}
Gini.index(frequency_1)
###Figure 5(b)###
pos_Gini <- rep(NA, 30)
neg_Gini <- rep(NA, 30)
for(i in 1:30){
	pos <- mean + 0.1*i*sd
	neg <- mean - 0.1*i*sd
	X_pos <- matrix(c(1, pos, 0), nrow=1, ncol=3) 
	X_neg <- matrix(c(1, neg, 0), nrow=1, ncol=3) 
	clr_y_pos_disc <- X_pos%*%original_est_res$beta_disc
	clr_y_neg_disc <- X_neg%*%original_est_res$beta_disc
	test2 <- exp(clr_y_pos_disc)
	test2_con <- approxfun(z, test2)
	normalize2 <- integrate(test2_con, lower = l_bound, upper = u_bound)$value
	test3 <- exp(clr_y_neg_disc)
	test3_con <- approxfun(z, test3)
	normalize3 <- integrate(test3_con, lower = l_bound, upper = u_bound)$value
	frequency_2 <- test2/normalize2
	frequency_3 <- test3/normalize3
	pos_Gini[i] <- Gini.index(frequency_2)
	neg_Gini[i] <- Gini.index(frequency_3)
	}
neg_Gini <- rev(neg_Gini)
Gini_seq <- c(neg_Gini, Gini.index(frequency_1), pos_Gini)
h_axis <- seq(-3, 3, 0.1)
#plot(h_axis, Gini_seq, type="l", lwd=2, xlab="a", ylab="Gini")
###############################

