#####Simulation study in Section 4.1#####
set.seed(123)	
library(fda)

#####Set 1, lognormal distribution#####
n <- 100	#Sample size
wid <- .001		#Accuracy
u_bound <- 2.8	#Upper bound of the domain
l_bound <- 0.2	#Lower bound of the domain
eta <- u_bound - l_bound	
z <- seq(l_bound, u_bound, wid)
number.z <- length(z)
number.basis <- 2*10	#Numbr of basis. This basis is used to generate an infinite-dimensional standard Gaussian variable.
root.lambda <- rep(2/3, number.basis)	#This is used to generate an infinite-dimensional standard Gaussian variable.

norm_L2_disc <- function(x){		#L2 norm
	x <- x^2
	integ <- wid*sum((1/2)*sum(x[1], x[number.z]), sum(x[2:(number.z -1)]))
	norm <- integ^(1/2)
	return(norm)
	}
est <- function(Y_mat, X_var, X_var_ast){			#The least square estimate
	beta_disc <- solve(crossprod(X_var), crossprod(X_var, Y_mat))
	beta_ast_disc <- solve(crossprod(X_var_ast), crossprod(X_var_ast, Y_mat))
	clr_y_hat_disc <- X_var%*%beta_disc
	clr_y_hat_ast_disc <- X_var_ast%*%beta_ast_disc
	error_disc <- matrix(0, ncol=length(z), nrow=n)
	for(i in 1:n){
		error_disc[i, ] <- Y_mat[i, ] - clr_y_hat_disc[i, ]
	}
	error_ast_disc <- matrix(0, ncol=length(z), nrow=n)
	for(i in 1:n){
		error_ast_disc[i, ] <- Y_mat[i, ] - clr_y_hat_ast_disc[i, ]
	}
	norm_error <- numeric(n)
	for(i in 1:n){
		norm_error[i] <- norm_L2_disc(error_disc[i,])^2
	}
	sum_error <- sum(norm_error)
	norm_error_ast <- numeric(n)
	for(i in 1:n){
		norm_error_ast[i] <- norm_L2_disc(error_ast_disc[i, ])^2
	}
	sum_ast_error <- sum(norm_error_ast)
	test_lambda <- sum_ast_error - sum_error
	return(list(beta_disc=beta_disc, clr_y_hat_disc=clr_y_hat_disc, beta_ast_disc=beta_ast_disc, clr_y_hat_ast_disc=clr_y_hat_ast_disc, 
			error_disc=error_disc, error_ast_disc=error_ast_disc, sum_error=sum_error, sum_ast_error=sum_ast_error, test_lambda=test_lambda))
}
#####Samples generated under the hypothesis H^1_0#####
H.norm <- as.list(rep(NA, n))		#Random variables drawn from H-valued Gaussian distribution. They are approximated by finite Fourier series.
for(i in 1:n){
	v.norm <- rnorm(number.basis, 0, .02)
	F.basis <- create.fourier.basis(c(l_bound, u_bound), nbasis=(number.basis + 1), dropind=1)		#Fourier series
	eval.F.basis <- eval.basis(z, F.basis)
	coeff <- root.lambda * v.norm
	H.norm[[i]] <- eval.F.basis %*% coeff
	}
rm.var <- runif(n, -1, 4)
Inv.var <- exp(rm.var)		#Inverse of the variance, which is also used as explanatory variable.
density.lognormal <- matrix(NA, ncol=length(z), nrow=n)
log.density <- matrix(NA, ncol=length(z), nrow=n)
norm.const <- rep(NA, n)
norm.const.log <- rep(NA, n)
clr.lognormal <- matrix(NA, ncol=length(z), nrow=n)
clr_y_matrix <- matrix(NA, ncol=length(z), nrow=n)
for(i in 1:n){
	density.lognormal[i,] <- dlnorm(z, meanlog = 0, sdlog = sqrt(1/Inv.var[i]), log = FALSE)		#Density function of lognormal distribution.
	norm.const[i] <- wid*sum((1/2)*sum(density.lognormal[i,1], density.lognormal[i,number.z]), sum(density.lognormal[i,2:(number.z -1)]))	#Integration
	density.lognormal[i,] <- density.lognormal[i,]/norm.const[i]	#Normalization
	log.density[i,] <- log(density.lognormal[i,])
	norm.const.log[i] <- wid*sum((1/2)*sum(log.density[i,1], log.density[i,number.z]), sum(log.density[i,2:(number.z -1)]))
	clr.lognormal[i,] <- log.density[i,] - (1/eta)*norm.const.log[i]		#Clr transformation
	clr_y_matrix[i,] <- clr.lognormal[i,] + H.norm[[i]]
	}
###Constants###
x_1 <- rep(1, n)
x_2 <- Inv.var
temp <- cbind(x_1, x_2)
X_ex <- as.matrix(temp)
temp_ast <- cbind(x_1)
X_ex_ast <- matrix(temp_ast, ncol=1, nrow=n)
###Estimation###
original_est_res <- est(clr_y_matrix, X_ex, X_ex_ast)
###Figure 1###
graph.density.lognormal <- matrix(NA, ncol=length(z), nrow=26)
graph.norm.const <- rep(NA, 26)
for(i in 1:26){
	graph.density.lognormal[i,] <- dlnorm(z, meanlog = 0, sdlog = sqrt(1/exp(-1.2+0.2*i)), log = FALSE)
	graph.norm.const[i] <- wid*sum((1/2)*sum(graph.density.lognormal[i,1], graph.density.lognormal[i,number.z]), sum(graph.density.lognormal[i,2:(number.z -1)]))	#Integration
	graph.density.lognormal[i,] <- graph.density.lognormal[i,]/graph.norm.const[i]
	}
plot(z, graph.density.lognormal[1,], xlim=c(l_bound, u_bound), ylim=c(0, 2.8), type="l", lwd=2, xlab="z", ylab="Density")	#Figure 1(a)
for(i in 2:26){
	lines(z, graph.density.lognormal[i,], lwd=2, col=gray(0.03*i))
	}
X.est <- matrix(c(1, exp(-1)), nrow=1, ncol=2)
clr_y_est <- X.est%*%original_est_res$beta_disc
Density.est <- exp(clr_y_est)
nor.est <- wid*sum((1/2)*sum(Density.est[1], Density.est[number.z]), sum(Density.est[2:(number.z -1)]))
normal.est <- Density.est/nor.est
plot(z, normal.est, xlim=c(l_bound, u_bound), ylim=c(0, 2.8), type="l", lwd=2, xlab="z", ylab="Density")	#Figure 1(b)
for(i in 2:26){
	X.est <- matrix(c(1, exp(-1.2+0.2*i)), nrow=1, ncol=2)
	clr_y_est <- X.est%*%original_est_res$beta_disc
	Density.est <- exp(clr_y_est)
	nor.est <- wid*sum((1/2)*sum(Density.est[1], Density.est[number.z]), sum(Density.est[2:(number.z -1)]))
	normal.est <- Density.est/nor.est
	lines(z, normal.est, lwd=2, col=gray(0.03*i))
	}
###Estimation by FDA###
y.matrix <- matrix(NA, ncol=length(z), nrow=n)
for(i in 1:n){
	Density.est <- exp(clr_y_matrix[i,])
	nor.est <- wid*sum((1/2)*sum(Density.est[1], Density.est[number.z]), sum(Density.est[2:(number.z -1)]))
	y.matrix[i,] <- Density.est/nor.est
	}
L.2.est <- est(y.matrix, X_ex, X_ex_ast)

X.est <- matrix(c(1, exp(-1)), nrow=1, ncol=2)
y.est <- X.est%*%L.2.est$beta_disc
plot(z, y.est, xlim=c(l_bound, u_bound), ylim=c(-.3, 2.5), type="l", lwd=2, xlab="z", ylab="Density")		#Figure 1(c)
for(i in 2:26){
	X.est <- matrix(c(1, exp(-1.2+0.2*i)), nrow=1, ncol=2)
	y.est <- X.est%*%L.2.est$beta_disc
	lines(z, y.est, lwd=2, col=gray(0.03*i))
	}
############################

#####Set 2, gamma distribution#####
n <- 100	#Sample size
wid <- .001		#Accuracy
u_bound <- 2.8	#Upper bound
l_bound <- 0.2	#Lower bound
eta <- u_bound - l_bound	
z <- seq(l_bound, u_bound, wid)

number.z <- length(z)
number.basis <- 2*10
root.lambda <- rep(2/3, number.basis)

#####Gamma#####
H.norm <- as.list(rep(NA, n))
for(i in 1:n){
	v.norm <- rnorm(number.basis, 0, .02)
	F.basis <- create.fourier.basis(c(l_bound, u_bound), nbasis=(number.basis + 1), dropind=1)
	eval.F.basis <- eval.basis(z, F.basis)
	coeff <- root.lambda * v.norm
	H.norm[[i]] <- eval.F.basis %*% coeff
	}
alpha <- runif(n, 1.7, 4.5)		#Shape parameter drawn from an uniform distribution.
beta <- runif(n, 2.2, 5.0)		#Rate parameter drawn from an uniform distribution.
density.gamma <- matrix(NA, ncol=length(z), nrow=n)
log.density <- matrix(NA, ncol=length(z), nrow=n)
norm.const <- rep(NA, n)
norm.const.log <- rep(NA, n)
clr.gamma <- matrix(NA, ncol=length(z), nrow=n)
clr_y_matrix <- matrix(NA, ncol=length(z), nrow=n)
for(i in 1:n){
	density.gamma[i,] <- dgamma(z, shape=alpha[i], rate=beta[i], log = FALSE)
	norm.const[i] <- wid*sum((1/2)*sum(density.gamma[i,1], density.gamma[i,number.z]), sum(density.gamma[i,2:(number.z -1)]))
	density.gamma[i,] <- density.gamma[i,]/norm.const[i]
	log.density[i,] <- log(density.gamma[i,])
	norm.const.log[i] <- wid*sum((1/2)*sum(log.density[i,1], log.density[i,number.z]), sum(log.density[i,2:(number.z -1)]))
	clr.gamma[i,] <- log.density[i,] - (1/eta)*norm.const.log[i]
	clr_y_matrix[i,] <- clr.gamma[i,] + H.norm[[i]]
	}
###Constants###
x_1 <- rep(1, n)
x_2 <- alpha
x_3 <- beta
temp <- cbind(x_1, x_2, x_3)
X_ex <- as.matrix(temp)
temp_ast <- cbind(x_1, x_3)
X_ex_ast <- as.matrix(temp_ast)
###Estimation###
original_est_res <- est(clr_y_matrix, X_ex, X_ex_ast)
###Figure 2###
graph.density.gamma <- matrix(NA, ncol=length(z), nrow=100)
graph.norm.const <- rep(NA, 100)
for(i in 1:10){
	for(j in 1:10){
		graph.density.gamma[10*(i-1)+j,] <- dgamma(z, shape=1.5+0.3*i, rate=2+0.3*j, log = FALSE)
		graph.norm.const[10*(i-1)+j] <- wid*sum((1/2)*sum(graph.density.gamma[10*(i-1)+j,1], graph.density.gamma[10*(i-1)+j,number.z]), sum(graph.density.gamma[10*(i-1)+j,2:(number.z -1)]))	#Integration
		graph.density.gamma[10*(i-1)+j,] <- graph.density.gamma[10*(i-1)+j,]/graph.norm.const[10*(i-1)+j]
		}
	}
plot(z, graph.density.gamma[1,], xlim=c(l_bound, u_bound), ylim=c(0, 1.6), type="l", lwd=2, xlab="z", ylab="Density")	#Figure 2(a)
for(i in 2:100){
	lines(z, graph.density.gamma[i,], lwd=2, col=gray(0.01*i))
	}
X.est <- matrix(c(1, 1.7, 2.2), nrow=1, ncol=3)
clr_y_est <- X.est%*%original_est_res$beta_disc
Density.est <- exp(clr_y_est)
nor.est <- wid*sum((1/2)*sum(Density.est[1], Density.est[number.z]), sum(Density.est[2:(number.z -1)]))
gamma.est <- Density.est/nor.est
plot(z, gamma.est, xlim=c(l_bound, u_bound), ylim=c(0, 1.6), type="l", lwd=2, xlab="z", ylab="Density")	#Figure 2(b)
for(i in 1:10){
	for(j in 1:10){
		X.est <- matrix(c(1, 1.5+0.3*i, 2+0.3*j), nrow=1, ncol=3)
		clr_y_est <- X.est%*%original_est_res$beta_disc
		Density.est <- exp(clr_y_est)
		nor.est <- wid*sum((1/2)*sum(Density.est[1], Density.est[number.z]), sum(Density.est[2:(number.z -1)]))
		gamma.est <- Density.est/nor.est
		lines(z, gamma.est, lwd=2, col=gray(0.1*(i-1)+0.01*j))
		}
	}
###Estimation by FDA###
gamma.y.matrix <- matrix(NA, ncol=length(z), nrow=n)
for(i in 1:n){
	Density.est <- exp(clr_y_matrix[i,])
	nor.est <- wid*sum((1/2)*sum(Density.est[1], Density.est[number.z]), sum(Density.est[2:(number.z -1)]))
	gamma.y.matrix[i,] <- Density.est/nor.est
	}
L.2.est <- est(gamma.y.matrix, X_ex, X_ex_ast)

X.est <- matrix(c(1, 1.7, 2.2), nrow=1, ncol=3)
y.est <- X.est%*%L.2.est$beta_disc
plot(z, y.est, xlim=c(l_bound, u_bound), ylim=c(-.3, 1.7), type="l", lwd=2, xlab="z", ylab="Density")		#Figure 2(c)
for(i in 1:10){
	for(j in 1:10){
		X.est <- matrix(c(1, 1.5+0.3*i, 2+0.3*j), nrow=1, ncol=3)
		gamma.y.est <- X.est%*%L.2.est$beta_disc
		lines(z, gamma.y.est, lwd=2, col=gray(0.1*(i-1)+0.01*j))
		}	
	}

